VERSION 5.00
Begin VB.Form frmMain 
   Caption         =   "vbCG - Ejemplo 2 (Vertex & Fragment Program)"
   ClientHeight    =   3195
   ClientLeft      =   60
   ClientTop       =   345
   ClientWidth     =   5190
   Icon            =   "frmMain.frx":0000
   LinkTopic       =   "Form1"
   ScaleHeight     =   213
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   346
   StartUpPosition =   3  'Windows Default
End
Attribute VB_Name = "frmMain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit
'-------------------------------------------------------------------------------------------
'Author           : Paul Dahuach
'Year             : 2008
'Legal            : Do whatever you want with it, send me some feedback if you want.
'More Examples    : http://www.pauldahuach.com.ar/program_en.htm
'Dependencies     : vbGL.tlb (http://home.pacific.net.hk/~edx/tlb.htm)
'-------------------------------------------------------------------------------------------

Private myCgContext             As Long
Private myCgVertexProfile       As Long
Private myCgVertexProgram       As Long
Private myCgFragmentProfile     As Long
Private myCgFragmentProgram     As Long
Private blnUnload               As Boolean
Private hRC                     As Long
Private AspectRadio             As Double
Private pfd                     As PIXELFORMATDESCRIPTOR
Private iFormat                 As Long

Const myVertexPrgFileName = "C2E1v_red.cg" & vbNullChar
Const myVertexProgramName = "C2E1v_red" & vbNullChar
Const myFragmentPrgFileName = "C2E2f_passthru.cg"
Const myFragmentProgramName = "C2E2f_passthru"

Private Sub Form_Load()
    If blnUnload Then Exit Sub
    pfd.nSize = Len(pfd)
    pfd.nVersion = 1
    pfd.dwFlags = PFD_DRAW_TO_WINDOW Or _
                  PFD_SUPPORT_OPENGL Or _
                  PFD_DOUBLEBUFFER
    pfd.iPixelType = PFD_TYPE_RGBA
    pfd.cColorBits = 32
    pfd.cDepthBits = 24
    pfd.cStencilBits = 8
    pfd.cAccumBits = 0
    pfd.iLayerType = PFD_MAIN_PLANE
    iFormat = ChoosePixelFormat(hDC, pfd)
    
    SetPixelFormat hDC, iFormat, pfd
    
    hRC = wglCreateContext(hDC)
    wglMakeCurrent hDC, hRC
    
    Me.Show
    Me.Refresh
    
    glShadeModel GL_SMOOTH
    glClearColor 0.25, 0.25, 0.25, 1
    glClearDepth 1
    glEnable GL_DEPTH_TEST
    glDepthFunc GL_LEQUAL
    glHint GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST
    
    myCgContext = cgCreateContext()
    checkForCgError "creando el contexto Cg"
    
    myCgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX)
    checkForCgError "obteniendo el ultimo perfil de vertices"
    
    cgGLSetOptimalOptions myCgVertexProfile
    checkForCgError "seleccionando el ultimo perfil de vertices"
    
    myCgVertexProgram = cgCreateProgramFromFile( _
                            myCgContext, _
                            CG_SOURCE, _
                            myVertexPrgFileName, _
                            myCgVertexProfile, _
                            myVertexProgramName, _
                            vbNullString)
    checkForCgError "creando el Vertex Program desde un archivo"
    
    cgGLLoadProgram myCgVertexProgram
    checkForCgError "cargando el Vertex Program"
    
    myCgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT)
    checkForCgError "obteniendo el ultimo perfil de framentos"
    
    cgGLSetOptimalOptions myCgFragmentProfile
    checkForCgError "seleccionando el ultimo perfil de framentos"

    myCgFragmentProgram = cgCreateProgramFromFile( _
                             myCgContext, _
                             CG_SOURCE, _
                             myFragmentPrgFileName, _
                             myCgFragmentProfile, _
                             myFragmentProgramName, _
                             vbNullString)
    checkForCgError "creando el Fragment Program desde un archivo"
    
    cgGLLoadProgram myCgFragmentProgram
    checkForCgError "cargando el Fragment Program"
    
    Do Until GetAsyncKeyState(vbKeyEscape) <> 0 Or blnUnload
       DoEvents
       If blnUnload Then Exit Do
       
       glClear GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT
       
       cgGLBindProgram myCgVertexProgram
       checkForCgError "enlazando el Vertex Program"

       cgGLEnableProfile myCgVertexProfile
       checkForCgError "activando el perfil de Vertices"

       cgGLBindProgram myCgFragmentProgram
       checkForCgError "enlazando el Fragment Program"

       cgGLEnableProfile myCgFragmentProfile
       checkForCgError "activando el perfil de Fragmentos"
       
       Call drawStars

       cgGLDisableProfile myCgVertexProfile
       checkForCgError "desactivando el perfil de Vertices"
       
       cgGLDisableProfile myCgFragmentProfile
       checkForCgError "desactivando el perfil de Fragmentos"
       
       SwapBuffers Me.hDC
       
    Loop
    
    cgDestroyProgram myCgVertexProgram
    cgDestroyProgram myCgFragmentProgram
    cgDestroyContext myCgContext
    wglMakeCurrent 0, 0
    wglDeleteContext hRC
    
    End

End Sub

Private Sub drawStar(x As Single, y As Single, starPoints As Long, Radio As Single, r As Single)
    Dim i                 As Long
    Dim angle             As Double
    Dim piOverStarPoints  As Double
    
    piOverStarPoints = 3.14159 / starPoints
    angle = 0#
    glBegin GL_TRIANGLE_FAN
      glVertex2f x, y ' El centro de la estrella
      ' Emitir los vertices exteriores para los puntos de la estrella.
      For i = 0 To starPoints - 1
          glVertex2f x + Radio * Cos(angle), y + Radio * Sin(angle)
          angle = angle + piOverStarPoints
          glVertex2f x + r * Cos(angle), y + r * Sin(angle)
          angle = angle + piOverStarPoints
      Next
      ' Terminar repitiendo el primer vertice exterior de la estrella.
      angle = 0
      glVertex2f x + Radio * Cos(angle), y + Radio * Sin(angle)
    glEnd
End Sub

Sub drawStars()
    '                     star    outer   inner
    '        x      y     Points  radius  radius
    '       =====  =====  ======  ======  ======
    drawStar -0.1, 0, 5, 0.5, 0.2
    drawStar -0.84, 0.1, 5, 0.3, 0.12
    drawStar 0.92, -0.5, 5, 0.25, 0.11
    drawStar 0.3, 0.97, 5, 0.3, 0.1
    drawStar 0.94, 0.3, 5, 0.5, 0.2
    drawStar -0.97, -0.8, 5, 0.6, 0.2
End Sub

Private Sub Form_Unload(Cancel As Integer)
    blnUnload = True
End Sub

Private Sub Form_Resize()
    If Me.WindowState = vbMinimized Then Exit Sub
    AspectRadio = Me.ScaleWidth / Me.ScaleHeight
    glViewport 0, 0, Me.ScaleWidth, Me.ScaleHeight
    glMatrixMode GL_PROJECTION
    glLoadIdentity
    gluPerspective 45#, AspectRadio + 0, 0.1, 1000#
    glMatrixMode GL_MODELVIEW
    glLoadIdentity
End Sub

Private Sub LogToFile(texto As String)
    Dim ff As Integer
    ff = FreeFile
    Open App.Path & "\Log.txt" For Append As #ff
    Print #ff, texto
    Close #ff
End Sub

Public Sub checkForCgError(contexto As String)
    Dim lngCGerror As Long
    Dim lpszError  As Long
    lngCGerror = cgGetError
    If lngCGerror <> CG_NO_ERROR _
       Then
           LogToFile contexto & ": " & lpszError
           If lngCGerror = CG_COMPILER_ERROR _
              Then
                  LogToFile cgGetLastListing(myCgContext)
           End If
           End
    End If
End Sub
